#-*- coding:utf-8 -*-

#  Pybik -- A 3 dimensional magic cube game.
#  Copyright (C) 2009  B. Clausius
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License,  or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.


# Ported from GNUbik
# Original filename: glarea.c
# Original copyright and license:
#/*
#  GNUbik -- A 3 dimensional magic cube game.
#  Copyright (C) 2003, 2004  John Darrington
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License,  or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#*/


import sys, os
import traceback

import drwBlock
import cube
import ui
import select_
import glarea_common
import widget_set
#import quarternion
import cursors
import textures
import colour_sel
from confstore import confstore

from OpenGL import GL as gl
from OpenGL import GLU as glu
from OpenGL import GLX as glx
from OpenGL import GLUT as glut

import gobject
import gtk
from gtk import gdk
from gtk import gtkgl
from gtk import gdkgl


glwidget = None
cursorAngle = 0.0

#@debug
def glarea_init(argv):
    glut.glutInit(len(argv)-1, *argv[1:])


def re_initialize_glarea ():
    graphics_area_init (glwidget)


# Resize callback.  
def resize (widget, event):
    height = event.height
    width =  event.width
    glcontext = gtkgl.widget_get_gl_context (widget)
    gldrawable = gtkgl.widget_get_gl_drawable (widget)

    if not gdkgl.Drawable.gl_begin (gldrawable,  glcontext):
        return False

    glx.glXWaitX ()
    min_dim = min(width, height)

    # Ensure that cube is always the same proportions 
    gl.glViewport ((width-min_dim) / 2,  (height-min_dim) / 2,  min_dim,  min_dim)

    return False


#@debug
def graphics_area_init(widget):
    glcontext = gtkgl.widget_get_gl_context (widget)
    gldrawable = gtkgl.widget_get_gl_drawable (widget)

    if not gldrawable.gl_begin (glcontext):
        print "Cannot initialise gl drawable"
        return

    glarea_common.lighting_init(confstore.lighting)
    textures.texInit ()
    glarea_common.projection_init (0)
    glarea_common.modelViewInit ()
    colour_sel.set_the_colours()


redisplayPending = False
setcursorPending = False

def postRedisplay():
    global redisplayPending
    if not redisplayPending:
        gobject.idle_add (handleRedisplay,  glwidget)
        redisplayPending = 1


def postSetCursor():
    global setcursorPending
    if not setcursorPending:
        gobject.idle_add(handleSetCursor,  glwidget)
        setcursorPending = 1


# Error string display.  This is always called by a macro
# wrapper,  to set the file and line_no opts parameters 
def error_check (string):
    err_state = gl.glGetError ()
    if gl.GL_NO_ERROR != err_state:
        try:
            frame = sys._getframe(1)
        except ValueError:
            pass
        else:
            print "%s:%d %s:  %s\n" % (os.path.basename(frame.f_code.co_filename),
                                        frame.f_lineno,
                                        string,
                                        glu.gluErrorString(err_state))


# Expose callback.  Just redraw the scene 
#@debug
def expose (unused_w, unused_event):
    handleRedisplay(glwidget)
    #return True


button_down = False

# Rotate the cube about the z axis (relative to the viewer ) 
def z_rotate (unused_w, event):
    ui.rotate_cube (2, not event.direction)
    return False


# Record the state of button 1 
def buttons (unused_w, event):
    # In GTK1-2,  buttons 4 and 5 mean mouse wheel scrolling 
    if event.button == 4:
        ui.rotate_cube (2, 1)

    if event.button == 5:
        ui.rotate_cube (2, 0)

    if event.button != 1:
        return False


    global button_down
    if event.type == gdk.BUTTON_PRESS:
        button_down = True
        select_.disableSelection ()
    elif event.type == gdk.BUTTON_RELEASE:
        select_.enableSelection ()
        button_down = False

    return False


last_mouse_x = -1
last_mouse_y = -1
def cube_orientate_mouse (unused_w, event):
    global last_mouse_x, last_mouse_y
    xmotion = 0
    ymotion = 0

    if not button_down:
        return False

    if select_.itemIsSelected ():
        return False


    if last_mouse_x >= 0:
        xmotion = event.x - last_mouse_x

    if last_mouse_y >= 0:
        ymotion = event.y - last_mouse_y


    last_mouse_x = event.x
    last_mouse_y = event.y

    if ymotion > 0:
        ui.rotate_cube (0, 1)
    if ymotion < 0:
        ui.rotate_cube (0, 0)

    if xmotion > 0:
        ui.rotate_cube (1, 1)
    if xmotion < 0:
        ui.rotate_cube (1, 0)

    return False


def cube_orientate_keys (widget, event):
    shifted = 0

    if event.state & gdk.SHIFT_MASK:
        shifted = 1

    return ui.do_keysyms (event.keyval,  shifted)


# Input callback.  Despatch input events to approprite handlers 
def cube_controls (unused_widget, event, application):
    if event.type != gdk.BUTTON_PRESS:
        return True

    ui.mouse (event.button, application)

    return True


# Pair of mutually co-operating funcs to handle redisplays,
# avoiding unnecessary overhead.  For example when the window
# is covered by another one 

def handleRedisplay (glxarea):
    display (glxarea)
    global redisplayPending
    redisplayPending = 0
    return False

def handleSetCursor(glxarea):
    set_mouse_cursor (glxarea)
    global setcursorPending
    setcursorPending = 0
    return False



def set_mouse_cursor (glxarea):
    fg = gdk.Color(65535, 65535, 65535) # White.
    bg = gdk.Color(    0,     0,     0) # Black.

    if select_.itemIsSelected ():
        data_bits, mask_bits, height, width, hot_x, hot_y = cursors.get_cursor(cursorAngle)

        source = gdk.bitmap_create_from_data (None, data_bits, width,  height)
        mask = gdk.bitmap_create_from_data (None, mask_bits, width,  height)

        cursor = gdk.Cursor (source,  mask,  fg,  bg,  hot_x,  hot_y)
    else:
        cursor = gdk.Cursor (gdk.CROSSHAIR)

    glxarea.window.set_cursor (cursor)



# Reset the bit planes,  and render the scene 
def display (glxarea):
    glcontext = gtkgl.widget_get_gl_context (glxarea)
    gldrawable = gtkgl.widget_get_gl_drawable (glxarea)

    if not gdkgl.Drawable.make_current (gldrawable,  glcontext):
        print "Cannot set gl drawable current"
        return
    error_check("Error in display")

    glarea_common.projection_init (0)
    glarea_common.modelViewInit ()


    set_mouse_cursor (glxarea)


    gl.glClear (gl.GL_ACCUM_BUFFER_BIT)
    error_check("Error in display")

    for jitter in xrange(8):
        gl.glClear (gl.GL_COLOR_BUFFER_BIT | gl.GL_DEPTH_BUFFER_BIT)

        glarea_common.projection_init (jitter)
        glarea_common.modelViewInit ()

        error_check("Error in display")

        drwBlock.draw_cube()

        error_check("Error in display")

        gl.glAccum (gl.GL_ACCUM,  1.0/8.0)


    gl.glAccum (gl.GL_RETURN, 1.0)

    gdkgl.Drawable.swap_buffers (gldrawable)


def set_lighting(enable):
    (gl.glEnable if enable else gl.glDisable)(gl.GL_LIGHTING)
    postRedisplay()

def get_lighting():
    return gl.glIsEnabled(gl.GL_LIGHTING)


# Render the turn indicators on the scene.
# Axis is the axis.
@debug
def turn_indicator (axis, dir):
    #int i

    if not select_.itemIsSelected ():
        return

    # Don't  render invisible faces
    gl.glEnable (gl.GL_CULL_FACE)


    for location in (cube.get_cube_dimension()*2.0, cube.get_cube_dimension()*-2.0):
        # one each side of the cube

        # Inside is dark grey
        gl.glFrontFace (gl.GL_CCW)
        gl.glColor3f (0.7, 0.7, 0.7)
        drawInd (location,  axis,  dir)

        # Outside is light grey
        gl.glFrontFace (gl.GL_CW)
        gl.glColor3f (0.3, 0.3, 0.3)
        drawInd (location,  axis,  dir)


    # We reset this to avoid trouble later !
    gl.glFrontFace (gl.GL_CCW)


# Draw the little indicators to show which way the blocks turn
@debug
def drawInd (location, axis, dir):
    radius = cube.get_cube_dimension() / 2.0
    strip_width = 0.1 * cube.get_cube_dimension()
    arrowlength = 0.1

    circ = (0,0,0)

    for start in (0,120,240):
        stop = start + 100

        width = strip_width
        crit = ((stop-start) * (1-arrowlength) + start if dir else
                (stop-start) * arrowlength + start)


        gl.glBegin (gl.GL_TRIANGLE_STRIP)
        for theta in xrange(start, stop+1):
            if dir:
                if theta > crit:
                    width = strip_width * abs((stop - theta) / (stop - crit))
                else:
                    width = strip_width
            else:
                if theta < crit:
                    width = strip_width * abs((start - theta) / (start - crit))
                else:
                    width = strip_width
                
            circ[axis] = location + (width * ((theta%2)*2 - 1))
            theta_rad = theta * math.pi / 180.0
            circ[(axis + 1)%3] = radius * math.sin(theta_rad)
            circ[(axis + 2)%3] = radius * math.cos(theta_rad)
            gl.glVertex3fv (circ)
        
        gl.glEnd ()

