#-*- coding:utf-8 -*-

#  Pybik -- A 3 dimensional magic cube game.
#  Copyright (C) 2009  B. Clausius
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License,  or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.


# Ported from GNUbik
# Original filename: guile-hooks.c
# Original copyright and license:
#/*
#    Copyright (C) 2004  Dale Mellor
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 3 of the License,  or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#*/


import os
import sys

import config

import cube
import ui
import widget_set
import move_queue_m


# When a script runs,  the first cube movement that is requested by the script
# must flush the move queue after the place of current insertion; further move
# requests must be appended to the end of the move queue. This variable is used
# to track the first request.
moved = False


# This function is called from the menu when the user makes a selection. The
# data is a string which was registered with the menu system and gives the name
# of a procedure to execute.
@debug
def run_script(func, application):
    global moved
    moved = False

    func()
    if moved:
        application.update_statusbar()
        if not ui.request_play(application):
            ui.update_ui(application)


# Function callable from plugin which returns a structure
# reflecting the current state of the cube.
@debug
def plugin_cube_state():
    return cube.make_plugin_cube()


# The first time a script makes a move on the cube,  the move_queue must be
# truncated to the current place,  and the place is marked so that the viewer
# can rewind the effects of the script. This script performs the necessary
# preparations.
def start_moves_if_first():
    global moved
    if not moved:
        moved = 1
        ui.request_truncate_move_queue()
        ui.request_mark_move_queue()


# Function which, when called from plugins, causes one
# side of the cube to rotate on-screen.
@debug
def plugin_rotate_animated(moves):
    start_moves_if_first()
    
    for maxis,mslice,mdir in moves:
        move = move_queue_m.MoveData()
        move.axis = maxis
        move.slice = mslice
        move.dir = mdir
        ui.request_delayed_rotation(move)


# Function allowing a script to apply all its moves in one go to the cube,
# without creating animations on the display.
@debug
def plugin_run_moves():
    ui.request_fast_forward()


# Function to allow a plugin to display a message to the user.
@debug
def plugin_error_dialog(message):
    widget_set.error_dialog(widget_set.main_application_window, message)


# Function to scan the named directory for all files with a .py extension, and import them.
@debug
def read_script_directory(dir_name):
    script_items = []
    if os.path.isdir(dir_name):
        sys.path.insert(0, dir_name)
        for entry in os.listdir(dir_name):
            if ".py" == entry[-3:]:
                module_name = entry[:-3]
                #buffer = "%s/%s" % (dir_name, entry)
                import imp
                file_,path,desc = imp.find_module(module_name, [dir_name])
                try:
                    module = imp.load_module(module_name, file_, path, desc)
                    if DEBUG:
                        print 'import', module_name
                except SyntaxError, e:
                    print e
                except ImportError, e:
                    print e
                else:
                    for fname, func in plugin_funcs:
                        setattr(module, fname, func)
                    if hasattr(module,'scripts'):
                        script_items += module.scripts
                finally:
                    if file_:
                        file_.close()
    return script_items
                

plugin_funcs = [
    ("plugin_cube_state",       plugin_cube_state),
    ("plugin_rotate_animated",  plugin_rotate_animated),
    ("plugin_run_moves",        plugin_run_moves),
    ("plugin_error_dialog",     plugin_error_dialog),
    ('random',                  lambda x:cube.rand.randrange(x)),
]

# This function initializes the plugins for us, and once the plugins have
# all been imported, it returns the requested menu structure to the caller.
@debug
def startup_plugins():
    script_items = read_script_directory(config.SCRIPT_DIR)

    script_dir = os.path.join(config.data_dir(), 'scripts')
    script_items += read_script_directory(script_dir)

    return script_items

